@@ -1,10 +1,9 @@
 """ Field declarations for models """
+import six
 import inspect
 import json
-from boto.dynamodb2.fields import (RangeKey, AllIndex, KeysOnlyIndex,
-                                   IncludeIndex)
-from boto.dynamodb2.types import (NUMBER, STRING, BINARY, NUMBER_SET,
-                                  STRING_SET, BINARY_SET)
+from dynamo3 import (DynamoKey, LocalIndex, NUMBER, STRING, BINARY, NUMBER_SET,
+                     STRING_SET, BINARY_SET, is_null)
 from decimal import Decimal
 
 from .conditions import Condition
@@ -34,9 +33,12 @@ class Field(object):
         (default unicode)
     coerce : bool, optional
         Attempt to coerce the value if it's the incorrect type (default False)
-    check : callable, optional
-        A function that takes the value and returns True if the value is valid
-        (default None)
+    check : callable or list, optional
+        A function that takes the value and returns True if the value is valid.
+        May also be a list of such functions. (default None)
+    nullable : bool, optional
+        If false, will add a check (above) to ensure the value is not null
+        (default True).
     default : object, optional
         The default value for this field that will be set when creating a model
         (default None, except for ``set`` data types which default to set())
@@ -63,8 +65,9 @@ class Field(object):
     """
 
     def __init__(self, hash_key=False, range_key=False, index=None,
-                 data_type=unicode, coerce=False, check=None, default=NO_ARG):
-        if sum((hash_key, range_key)) > 1:
+                 data_type=six.text_type, coerce=False, check=None,
+                 nullable=True, default=NO_ARG):
+        if hash_key and range_key:
             raise ValueError("hash_key and range_key are mutually exclusive!")
         self.name = None
         self.model = None
@@ -81,14 +84,21 @@ class Field(object):
                 raise TypeError("Unrecognized data_type '%s'" % data_type)
             self.data_type = type_factory()
         self._coerce = coerce
-        self.check = check
+        self.check = []
+        if check is not None:
+            if hasattr(check, '__iter__'):
+                self.check = list(check)
+            else:
+                self.check = [check]
+        if not nullable:
+            self.check.append(lambda x: not is_null(x))
         self.hash_key = hash_key
         self.range_key = range_key
         self.subfields = []
         self.index = False
         self.index_name = None
-        self._boto_index = None
-        self._boto_index_kwargs = None
+        self._ddb_index = None
+        self._ddb_index_kwargs = None
         if default is NO_ARG:
             if self.is_set:
                 self.default = set()
@@ -99,23 +109,45 @@ class Field(object):
         if index:
             self.all_index(index)
 
-    def get_boto_index(self, hash_key):
-        """ Construct a boto index object from a hash key """
-        parts = [hash_key, RangeKey(self.name, data_type=self.ddb_data_type)]
-        return self._boto_index(self.index_name, parts=parts,
-                                **self._boto_index_kwargs)
+    def get_ddb_index(self):
+        """ Construct a dynamo local index object """
+        range_key = DynamoKey(self.name, data_type=self.ddb_data_type)
+        return self._ddb_index(self.index_name, range_key,
+                               **self._ddb_index_kwargs)
 
-    def _set_boto_index(self, name, boto_index, **kwargs):
+    def _set_ddb_index(self, name, factory, **kwargs):
         """ Set the type of index """
         if self.hash_key or self.range_key:
             raise ValueError("Cannot index the hash or range key!")
         if self.index:
             raise ValueError("Index is already set!")
         self.index_name = name
-        self._boto_index = boto_index
-        self._boto_index_kwargs = kwargs
+        self._ddb_index = factory
+        self._ddb_index_kwargs = kwargs
         self.index = True
 
+    def validate(self, obj):
+        """
+        Run the validation checks for this field on a model object.
+
+        Parameters
+        ----------
+        obj : :class:`~flywheel.models.Model`
+
+        Raises
+        ------
+        err : :class:`ValueError`
+            Raised if any of the checks fail.
+
+        """
+        if self.check is None:
+            return
+        val = self.resolve(obj)
+        for check in self.check:
+            if not check(val):
+                raise ValueError("Validation check on field %s failed for "
+                                 "value %r" % (self.name, val))
+
     def all_index(self, name):
         """
         Index this field and project all attributes
@@ -126,7 +158,7 @@ class Field(object):
             The name of the index
 
         """
-        self._set_boto_index(name, AllIndex)
+        self._set_ddb_index(name, LocalIndex.all)
         return self
 
     def keys_index(self, name):
@@ -139,7 +171,7 @@ class Field(object):
             The name of the index
 
         """
-        self._set_boto_index(name, KeysOnlyIndex)
+        self._set_ddb_index(name, LocalIndex.keys)
         return self
 
     def include_index(self, name, includes=None):
@@ -155,7 +187,7 @@ class Field(object):
 
         """
         includes = includes or []
-        self._set_boto_index(name, IncludeIndex, includes=includes)
+        self._set_ddb_index(name, LocalIndex.include, includes=includes)
         return self
 
     def coerce(self, value, force_coerce=None):
@@ -182,11 +214,6 @@ class Field(object):
         """ Return True if data type is a set """
         return self.ddb_data_type in (STRING_SET, NUMBER_SET, BINARY_SET)
 
-    @classmethod
-    def is_null(cls, value):
-        """ Return True if the value corresponds to null inside dynamo """
-        return value is None or value == set()
-
     def ddb_dump(self, value):
         """ Dump a value to its Dynamo format """
         if value is None:
@@ -211,8 +238,7 @@ class Field(object):
         """ Dump an overflow value to its Dynamo format """
         if val is None:
             return None
-        elif (isinstance(val, int) or isinstance(val, float) or
-              isinstance(val, long)):
+        elif isinstance(val, six.integer_types) or isinstance(val, float):
             return val
         elif isinstance(val, set):
             return val
@@ -223,7 +249,7 @@ class Field(object):
     def ddb_load_overflow(cls, val):
         """ Decode a value of an overflow field """
         if (isinstance(val, Decimal) or isinstance(val, float) or
-           isinstance(val, int) or isinstance(val, long)):
+                isinstance(val, six.integer_types)):
             if val % 1 == 0:
                 return int(val)
             return float(val)
@@ -242,7 +268,7 @@ class Field(object):
 
     @property
     def ddb_data_type(self):
-        """ Get the DynamoDB data type as used by boto """
+        """ Get the native DynamoDB data type """
         return self.data_type.ddb_data_type
 
     def can_resolve(self, fields):
@@ -430,8 +456,8 @@ class Composite(Field):
                              'check', 'coerce']))
         if unrecognized:
             raise TypeError("Unrecognized keyword args: %s" % unrecognized)
-        if len(args) < 2:
-            raise TypeError("Composite must consist of two or more fields")
+        if len(args) < 1:
+            raise TypeError("Composite must depend on at least one field")
         super(Composite, self).__init__(**kwargs)
         self.composite = True
         self.subfields = args
